import { localize } from '../../nls.js';
const minute = 60;
const hour = minute * 60;
const day = hour * 24;
const week = day * 7;
const month = day * 30;
const year = day * 365;
function fromNow(date, appendAgoLabel, useFullTimeWords, disallowNow) {
    if (typeof date !== 'number') {
        date = date.getTime();
    }
    const seconds = Math.round((( (new Date())).getTime() - date) / 1000);
    if (seconds < -30) {
        return ( localize(
            976,
            'in {0}',
            fromNow(( (new Date())).getTime() + seconds * 1000, false)
        ));
    }
    if (!disallowNow && seconds < 30) {
        return ( localize(977, 'now'));
    }
    let value;
    if (seconds < minute) {
        value = seconds;
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(978, '{0} second ago', value))
                    : ( localize(979, '{0} sec ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(980, '{0} seconds ago', value))
                    : ( localize(981, '{0} secs ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(982, '{0} second', value))
                    : ( localize(983, '{0} sec', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(984, '{0} seconds', value))
                    : ( localize(985, '{0} secs', value));
            }
        }
    }
    if (seconds < hour) {
        value = Math.floor(seconds / minute);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(986, '{0} minute ago', value))
                    : ( localize(987, '{0} min ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(988, '{0} minutes ago', value))
                    : ( localize(989, '{0} mins ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(990, '{0} minute', value))
                    : ( localize(991, '{0} min', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(992, '{0} minutes', value))
                    : ( localize(993, '{0} mins', value));
            }
        }
    }
    if (seconds < day) {
        value = Math.floor(seconds / hour);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(994, '{0} hour ago', value))
                    : ( localize(995, '{0} hr ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(996, '{0} hours ago', value))
                    : ( localize(997, '{0} hrs ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(998, '{0} hour', value))
                    : ( localize(999, '{0} hr', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(1000, '{0} hours', value))
                    : ( localize(1001, '{0} hrs', value));
            }
        }
    }
    if (seconds < week) {
        value = Math.floor(seconds / day);
        if (appendAgoLabel) {
            return value === 1
                ? ( localize(1002, '{0} day ago', value))
                : ( localize(1003, '{0} days ago', value));
        }
        else {
            return value === 1
                ? ( localize(1004, '{0} day', value))
                : ( localize(1005, '{0} days', value));
        }
    }
    if (seconds < month) {
        value = Math.floor(seconds / week);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(1006, '{0} week ago', value))
                    : ( localize(1007, '{0} wk ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(1008, '{0} weeks ago', value))
                    : ( localize(1009, '{0} wks ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(1010, '{0} week', value))
                    : ( localize(1011, '{0} wk', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(1012, '{0} weeks', value))
                    : ( localize(1013, '{0} wks', value));
            }
        }
    }
    if (seconds < year) {
        value = Math.floor(seconds / month);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(1014, '{0} month ago', value))
                    : ( localize(1015, '{0} mo ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(1016, '{0} months ago', value))
                    : ( localize(1017, '{0} mos ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(1018, '{0} month', value))
                    : ( localize(1019, '{0} mo', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(1020, '{0} months', value))
                    : ( localize(1021, '{0} mos', value));
            }
        }
    }
    value = Math.floor(seconds / year);
    if (appendAgoLabel) {
        if (value === 1) {
            return useFullTimeWords
                ? ( localize(1022, '{0} year ago', value))
                : ( localize(1023, '{0} yr ago', value));
        }
        else {
            return useFullTimeWords
                ? ( localize(1024, '{0} years ago', value))
                : ( localize(1025, '{0} yrs ago', value));
        }
    }
    else {
        if (value === 1) {
            return useFullTimeWords
                ? ( localize(1026, '{0} year', value))
                : ( localize(1027, '{0} yr', value));
        }
        else {
            return useFullTimeWords
                ? ( localize(1028, '{0} years', value))
                : ( localize(1029, '{0} yrs', value));
        }
    }
}
function fromNowByDay(date, appendAgoLabel, useFullTimeWords) {
    if (typeof date !== 'number') {
        date = date.getTime();
    }
    const todayMidnightTime = ( (new Date()));
    todayMidnightTime.setHours(0, 0, 0, 0);
    const yesterdayMidnightTime = ( (new Date(todayMidnightTime.getTime())));
    yesterdayMidnightTime.setDate(yesterdayMidnightTime.getDate() - 1);
    if (date > todayMidnightTime.getTime()) {
        return ( localize(1030, 'Today'));
    }
    if (date > yesterdayMidnightTime.getTime()) {
        return ( localize(1031, 'Yesterday'));
    }
    return fromNow(date, appendAgoLabel, useFullTimeWords);
}
function getDurationString(ms, useFullTimeWords) {
    const seconds = Math.abs(ms / 1000);
    if (seconds < 1) {
        return ( localize(1033, '{0}ms', ms));
    }
    if (seconds < minute) {
        return ( localize(1035, '{0}s', Math.round(ms) / 1000));
    }
    if (seconds < hour) {
        return ( localize(1037, '{0} mins', Math.round(ms / (1000 * minute))));
    }
    if (seconds < day) {
        return ( localize(1039, '{0} hrs', Math.round(ms / (1000 * hour))));
    }
    return ( localize(1040, '{0} days', Math.round(ms / (1000 * day))));
}
function toLocalISOString(date) {
    return date.getFullYear() +
        '-' + String(date.getMonth() + 1).padStart(2, '0') +
        '-' + String(date.getDate()).padStart(2, '0') +
        'T' + String(date.getHours()).padStart(2, '0') +
        ':' + String(date.getMinutes()).padStart(2, '0') +
        ':' + String(date.getSeconds()).padStart(2, '0') +
        '.' + (date.getMilliseconds() / 1000).toFixed(3).slice(2, 5) +
        'Z';
}
export { fromNow, fromNowByDay, getDurationString, toLocalISOString };
