import { ISingleEditOperation } from './editOperation.js';
import { Position } from './position.js';
import { Range } from './range.js';
import { TextLength } from './textLength.js';

declare class TextEdit {
    readonly edits: readonly SingleTextEdit[];
    static single(originalRange: Range, newText: string): TextEdit;
    static insert(position: Position, newText: string): TextEdit;
    constructor(edits: readonly SingleTextEdit[]);
    /**
     * Joins touching edits and removes empty edits.
     */
    normalize(): TextEdit;
    mapPosition(position: Position): Position | Range;
    mapRange(range: Range): Range;
    inverseMapPosition(positionAfterEdit: Position, doc: AbstractText): Position | Range;
    inverseMapRange(range: Range, doc: AbstractText): Range;
    apply(text: AbstractText): string;
    applyToString(str: string): string;
    inverse(doc: AbstractText): TextEdit;
    getNewRanges(): Range[];
}
declare class SingleTextEdit {
    readonly range: Range;
    readonly text: string;
    constructor(range: Range, text: string);
    get isEmpty(): boolean;
    static equals(first: SingleTextEdit, second: SingleTextEdit): boolean;
    toSingleEditOperation(): ISingleEditOperation;
    toEdit(): TextEdit;
    equals(other: SingleTextEdit): boolean;
}
declare abstract class AbstractText {
    abstract getValueOfRange(range: Range): string;
    abstract readonly length: TextLength;
    get endPositionExclusive(): Position;
    getValue(): string;
    getLineLength(lineNumber: number): number;
}

export { AbstractText, SingleTextEdit, TextEdit };
