import '../../../../../base/common/arrays.js';
import '../../../../../base/common/event.js';
import { Disposable } from '../../../../../base/common/lifecycle.js';
import { autorun } from '../../../../../base/common/observableInternal/autorun.js';
import '../../../../../base/common/observableInternal/derived.js';
import '../../../../../base/common/cancellation.js';
import '../../../../../base/common/observableInternal/utils.js';
import { firstNonWhitespaceIndex } from '../../../../../base/common/strings.js';
import { CursorColumns } from '../../../../common/core/cursorColumns.js';
import { RawContextKey } from '../../../../../platform/contextkey/common/contextkey.js';
import { localize } from '../../../../../nls.js';
import { bindContextKey } from '../../../../../platform/observable/common/platformObservableUtils.js';
class InlineCompletionContextKeys extends Disposable {
    static { this.inlineSuggestionVisible = ( (new RawContextKey('inlineSuggestionVisible', false, ( localize(9989, "Whether an inline suggestion is visible"))))); }
    static { this.inlineSuggestionHasIndentation = ( (new RawContextKey('inlineSuggestionHasIndentation', false, ( localize(9990, "Whether the inline suggestion starts with whitespace"))))); }
    static { this.inlineSuggestionHasIndentationLessThanTabSize = ( (new RawContextKey('inlineSuggestionHasIndentationLessThanTabSize', true, ( localize(
        9991,
        "Whether the inline suggestion starts with whitespace that is less than what would be inserted by tab"
    ))))); }
    static { this.suppressSuggestions = ( (new RawContextKey('inlineSuggestionSuppressSuggestions', undefined, ( localize(
        9992,
        "Whether suggestions should be suppressed for the current suggestion"
    ))))); }
    static { this.cursorInIndentation = ( (new RawContextKey('cursorInIndentation', false, ( localize(9993, "Whether the cursor is in indentation"))))); }
    static { this.hasSelection = ( (new RawContextKey('editor.hasSelection', false, ( localize(9994, "Whether the editor has a selection"))))); }
    static { this.cursorAtInlineEdit = ( (new RawContextKey('cursorAtInlineEdit', false, ( localize(9995, "Whether the cursor is at an inline edit"))))); }
    static { this.inlineEditVisible = ( (new RawContextKey('inlineEditIsVisible', false, ( localize(9996, "Whether an inline edit is visible"))))); }
    constructor(contextKeyService, model) {
        super();
        this.contextKeyService = contextKeyService;
        this.model = model;
        this.inlineCompletionVisible = InlineCompletionContextKeys.inlineSuggestionVisible.bindTo(this.contextKeyService);
        this.inlineCompletionSuggestsIndentation = InlineCompletionContextKeys.inlineSuggestionHasIndentation.bindTo(this.contextKeyService);
        this.inlineCompletionSuggestsIndentationLessThanTabSize = InlineCompletionContextKeys.inlineSuggestionHasIndentationLessThanTabSize.bindTo(this.contextKeyService);
        this.suppressSuggestions = InlineCompletionContextKeys.suppressSuggestions.bindTo(this.contextKeyService);
        this._register(bindContextKey(InlineCompletionContextKeys.inlineEditVisible, this.contextKeyService, reader => this.model.read(reader)?.stateInlineEdit.read(reader) !== undefined));
        this._register(autorun(reader => {
            const model = this.model.read(reader);
            const state = model?.state.read(reader);
            const isInlineCompletionVisible = !!state?.inlineCompletion && state?.primaryGhostText !== undefined && !state?.primaryGhostText.isEmpty();
            this.inlineCompletionVisible.set(isInlineCompletionVisible);
            if (state?.primaryGhostText && state?.inlineCompletion) {
                this.suppressSuggestions.set(state.inlineCompletion.inlineCompletion.source.inlineCompletions.suppressSuggestions);
            }
        }));
        this._register(autorun(reader => {
            const model = this.model.read(reader);
            let startsWithIndentation = false;
            let startsWithIndentationLessThanTabSize = true;
            const ghostText = model?.primaryGhostText.read(reader);
            if (!!model?.selectedSuggestItem && ghostText && ghostText.parts.length > 0) {
                const { column, lines } = ghostText.parts[0];
                const firstLine = lines[0];
                const indentationEndColumn = model.textModel.getLineIndentColumn(ghostText.lineNumber);
                const inIndentation = column <= indentationEndColumn;
                if (inIndentation) {
                    let firstNonWsIdx = firstNonWhitespaceIndex(firstLine);
                    if (firstNonWsIdx === -1) {
                        firstNonWsIdx = firstLine.length - 1;
                    }
                    startsWithIndentation = firstNonWsIdx > 0;
                    const tabSize = model.textModel.getOptions().tabSize;
                    const visibleColumnIndentation = CursorColumns.visibleColumnFromColumn(firstLine, firstNonWsIdx + 1, tabSize);
                    startsWithIndentationLessThanTabSize = visibleColumnIndentation < tabSize;
                }
            }
            this.inlineCompletionSuggestsIndentation.set(startsWithIndentation);
            this.inlineCompletionSuggestsIndentationLessThanTabSize.set(startsWithIndentationLessThanTabSize);
        }));
    }
}
export { InlineCompletionContextKeys };
