import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { localize } from '../../../../nls.js';
import { Extensions as Extensions$1 } from '../../../../platform/configuration/common/configurationRegistry.js';
import { Registry } from '../../../../platform/registry/common/platform.js';
import { RawContextKey } from '../../../../platform/contextkey/common/contextkey.js';
import { Extensions, workbenchConfigurationNodeBase } from '../../../common/configuration.js';
import { AccessibilitySignal } from '../../../../platform/accessibilitySignal/browser/accessibilitySignalService.js';
import { SPEECH_LANGUAGES } from '../../speech/common/speechService.js';
import { ISpeechService } from '../../speech/common/speechService.service.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { Event } from '../../../../base/common/event.js';
import { isDefined } from '../../../../base/common/types.js';
import { IProductService } from '../../../../platform/product/common/productService.service.js';
const accessibilityHelpIsShown = ( (new RawContextKey('accessibilityHelpIsShown', false, true)));
const accessibleViewIsShown = ( (new RawContextKey('accessibleViewIsShown', false, true)));
const accessibleViewSupportsNavigation = ( (new RawContextKey('accessibleViewSupportsNavigation', false, true)));
const accessibleViewVerbosityEnabled = ( (new RawContextKey('accessibleViewVerbosityEnabled', false, true)));
const accessibleViewGoToSymbolSupported = ( (new RawContextKey('accessibleViewGoToSymbolSupported', false, true)));
const accessibleViewOnLastLine = ( (new RawContextKey('accessibleViewOnLastLine', false, true)));
const accessibleViewCurrentProviderId = ( (new RawContextKey('accessibleViewCurrentProviderId', undefined, undefined)));
const accessibleViewInCodeBlock = ( (new RawContextKey('accessibleViewInCodeBlock', undefined, undefined)));
const accessibleViewContainsCodeBlocks = ( (new RawContextKey('accessibleViewContainsCodeBlocks', undefined, undefined)));
const accessibleViewHasUnassignedKeybindings = ( (new RawContextKey('accessibleViewHasUnassignedKeybindings', undefined, undefined)));
const accessibleViewHasAssignedKeybindings = ( (new RawContextKey('accessibleViewHasAssignedKeybindings', undefined, undefined)));
const baseVerbosityProperty = {
    type: 'boolean',
    default: true,
    tags: ['accessibility']
};
const accessibilityConfigurationNodeBase = ( (Object.freeze({
    id: 'accessibility',
    title: ( localize(4838, "Accessibility")),
    type: 'object'
})));
const soundFeatureBase = {
    'type': 'string',
    'enum': ['auto', 'on', 'off'],
    'default': 'auto',
    'enumDescriptions': [
        ( localize(4839, "Enable sound when a screen reader is attached.")),
        ( localize(4840, "Enable sound.")),
        ( localize(4841, "Disable sound."))
    ],
    tags: ['accessibility'],
};
const signalFeatureBase = {
    'type': 'object',
    'tags': ['accessibility'],
    additionalProperties: false,
    default: {
        sound: 'auto',
        announcement: 'auto'
    }
};
const announcementFeatureBase = {
    'type': 'string',
    'enum': ['auto', 'off'],
    'default': 'auto',
    'enumDescriptions': [
        ( localize(
            4842,
            "Enable announcement, will only play when in screen reader optimized mode."
        )),
        ( localize(4843, "Disable announcement."))
    ],
    tags: ['accessibility'],
};
const defaultNoAnnouncement = {
    'type': 'object',
    'tags': ['accessibility'],
    additionalProperties: false,
    'default': {
        'sound': 'auto',
    }
};
const configuration = {
    ...accessibilityConfigurationNodeBase,
    scope: 4 ,
    properties: {
        ["accessibility.verbosity.terminal" ]: {
            description: ( localize(
                4844,
                'Provide information about how to access the terminal accessibility help menu when the terminal is focused.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.diffEditor" ]: {
            description: ( localize(
                4845,
                'Provide information about how to navigate changes in the diff editor when it is focused.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.panelChat" ]: {
            description: ( localize(
                4846,
                'Provide information about how to access the chat help menu when the chat input is focused.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.inlineChat" ]: {
            description: ( localize(
                4847,
                'Provide information about how to access the inline editor chat accessibility help menu and alert with hints that describe how to use the feature when the input is focused.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.inlineCompletions" ]: {
            description: ( localize(
                4848,
                'Provide information about how to access the inline completions hover and Accessible View.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.keybindingsEditor" ]: {
            description: ( localize(
                4849,
                'Provide information about how to change a keybinding in the keybindings editor when a row is focused.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.notebook" ]: {
            description: ( localize(
                4850,
                'Provide information about how to focus the cell container or inner editor when a notebook cell is focused.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.hover" ]: {
            description: ( localize(
                4851,
                'Provide information about how to open the hover in an Accessible View.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.notification" ]: {
            description: ( localize(
                4852,
                'Provide information about how to open the notification in an Accessible View.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.emptyEditorHint" ]: {
            description: ( localize(
                4853,
                'Provide information about relevant actions in an empty text editor.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.replInputHint" ]: {
            description: ( localize(4854, 'Provide information about relevant actions For the Repl input.')),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.comments" ]: {
            description: ( localize(
                4855,
                'Provide information about actions that can be taken in the comment widget or in a file which contains comments.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.diffEditorActive" ]: {
            description: ( localize(4856, 'Indicate when a diff editor becomes the active editor.')),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.debug" ]: {
            description: ( localize(
                4857,
                'Provide information about how to access the debug console accessibility help dialog when the debug console or run and debug viewlet is focused. Note that a reload of the window is required for this to take effect.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.verbosity.walkthrough" ]: {
            description: ( localize(
                4858,
                'Provide information about how to open the walkthrough in an Accessible View.'
            )),
            ...baseVerbosityProperty
        },
        ["accessibility.accessibleView.closeOnKeyPress" ]: {
            markdownDescription: ( localize(
                4859,
                "On keypress, close the Accessible View and focus the element from which it was invoked."
            )),
            type: 'boolean',
            default: true
        },
        'accessibility.signalOptions.volume': {
            'description': ( localize(4860, "The volume of the sounds in percent (0-100).")),
            'type': 'number',
            'minimum': 0,
            'maximum': 100,
            'default': 70,
            'tags': ['accessibility']
        },
        'accessibility.signalOptions.debouncePositionChanges': {
            'description': ( localize(4861, "Whether or not position changes should be debounced")),
            'type': 'boolean',
            'default': false,
            'tags': ['accessibility']
        },
        'accessibility.signalOptions.experimental.delays.general': {
            'type': 'object',
            'description': 'Delays for all signals besides error and warning at position',
            'additionalProperties': false,
            'properties': {
                'announcement': {
                    'description': ( localize(4862, "The delay in milliseconds before an announcement is made.")),
                    'type': 'number',
                    'minimum': 0,
                    'default': 3000
                },
                'sound': {
                    'description': ( localize(4863, "The delay in milliseconds before a sound is played.")),
                    'type': 'number',
                    'minimum': 0,
                    'default': 400
                }
            },
            'tags': ['accessibility']
        },
        'accessibility.signalOptions.experimental.delays.warningAtPosition': {
            'type': 'object',
            'additionalProperties': false,
            'properties': {
                'announcement': {
                    'description': ( localize(
                        4864,
                        "The delay in milliseconds before an announcement is made when there's a warning at the position."
                    )),
                    'type': 'number',
                    'minimum': 0,
                    'default': 3000
                },
                'sound': {
                    'description': ( localize(
                        4865,
                        "The delay in milliseconds before a sound is played when there's a warning at the position."
                    )),
                    'type': 'number',
                    'minimum': 0,
                    'default': 1000
                }
            },
            'tags': ['accessibility']
        },
        'accessibility.signalOptions.experimental.delays.errorAtPosition': {
            'type': 'object',
            'additionalProperties': false,
            'properties': {
                'announcement': {
                    'description': ( localize(
                        4866,
                        "The delay in milliseconds before an announcement is made when there's an error at the position."
                    )),
                    'type': 'number',
                    'minimum': 0,
                    'default': 3000
                },
                'sound': {
                    'description': ( localize(
                        4867,
                        "The delay in milliseconds before a sound is played when there's an error at the position."
                    )),
                    'type': 'number',
                    'minimum': 0,
                    'default': 1000
                }
            },
            'tags': ['accessibility']
        },
        'accessibility.signals.lineHasBreakpoint': {
            ...signalFeatureBase,
            'description': ( localize(
                4868,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when the active line has a breakpoint."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4869, "Plays a sound when the active line has a breakpoint.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4870, "Announces when the active line has a breakpoint.")),
                    ...announcementFeatureBase
                },
            },
        },
        'accessibility.signals.lineHasInlineSuggestion': {
            ...defaultNoAnnouncement,
            'description': ( localize(
                4871,
                "Plays a sound / audio cue when the active line has an inline suggestion."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4872, "Plays a sound when the active line has an inline suggestion.")),
                    ...soundFeatureBase,
                    'default': 'off'
                }
            }
        },
        'accessibility.signals.lineHasError': {
            ...signalFeatureBase,
            'description': ( localize(
                4873,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when the active line has an error."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4874, "Plays a sound when the active line has an error.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4875, "Announces when the active line has an error.")),
                    ...announcementFeatureBase,
                    default: 'off'
                },
            },
        },
        'accessibility.signals.lineHasFoldedArea': {
            ...signalFeatureBase,
            'description': ( localize(
                4876,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - the active line has a folded area that can be unfolded."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4877,
                        "Plays a sound when the active line has a folded area that can be unfolded."
                    )),
                    ...soundFeatureBase,
                    default: 'off'
                },
                'announcement': {
                    'description': ( localize(
                        4878,
                        "Announces when the active line has a folded area that can be unfolded."
                    )),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.lineHasWarning': {
            ...signalFeatureBase,
            'description': ( localize(
                4879,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when the active line has a warning."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4880, "Plays a sound when the active line has a warning.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4881, "Announces when the active line has a warning.")),
                    ...announcementFeatureBase,
                    default: 'off'
                },
            },
        },
        'accessibility.signals.positionHasError': {
            ...signalFeatureBase,
            'description': ( localize(
                4882,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when the active line has a warning."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4883, "Plays a sound when the active line has a warning.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4884, "Announces when the active line has a warning.")),
                    ...announcementFeatureBase,
                    default: 'on'
                },
            },
        },
        'accessibility.signals.positionHasWarning': {
            ...signalFeatureBase,
            'description': ( localize(
                4885,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when the active line has a warning."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4886, "Plays a sound when the active line has a warning.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4887, "Announces when the active line has a warning.")),
                    ...announcementFeatureBase,
                    default: 'on'
                },
            },
        },
        'accessibility.signals.onDebugBreak': {
            ...signalFeatureBase,
            'description': ( localize(
                4888,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when the debugger stopped on a breakpoint."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4889, "Plays a sound when the debugger stopped on a breakpoint.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4890, "Announces when the debugger stopped on a breakpoint.")),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.noInlayHints': {
            ...signalFeatureBase,
            'description': ( localize(
                4891,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when trying to read a line with inlay hints that has no inlay hints."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4892,
                        "Plays a sound when trying to read a line with inlay hints that has no inlay hints."
                    )),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(
                        4893,
                        "Announces when trying to read a line with inlay hints that has no inlay hints."
                    )),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.taskCompleted': {
            ...signalFeatureBase,
            'description': ( localize(
                4894,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a task is completed."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4895, "Plays a sound when a task is completed.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4896, "Announces when a task is completed.")),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.taskFailed': {
            ...signalFeatureBase,
            'description': ( localize(
                4897,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a task fails (non-zero exit code)."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4898, "Plays a sound when a task fails (non-zero exit code).")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4899, "Announces when a task fails (non-zero exit code).")),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.terminalCommandFailed': {
            ...signalFeatureBase,
            'description': ( localize(
                4900,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a terminal command fails (non-zero exit code) or when a command with such an exit code is navigated to in the accessible view."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4901,
                        "Plays a sound when a terminal command fails (non-zero exit code) or when a command with such an exit code is navigated to in the accessible view."
                    )),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(
                        4902,
                        "Announces when a terminal command fails (non-zero exit code) or when a command with such an exit code is navigated to in the accessible view."
                    )),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.terminalCommandSucceeded': {
            ...signalFeatureBase,
            'description': ( localize(
                4903,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a terminal command succeeds (zero exit code) or when a command with such an exit code is navigated to in the accessible view."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4904,
                        "Plays a sound when a terminal command succeeds (zero exit code) or when a command with such an exit code is navigated to in the accessible view."
                    )),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(
                        4905,
                        "Announces when a terminal command succeeds (zero exit code) or when a command with such an exit code is navigated to in the accessible view."
                    )),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.terminalQuickFix': {
            ...signalFeatureBase,
            'description': ( localize(
                4906,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when terminal Quick Fixes are available."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4907, "Plays a sound when terminal Quick Fixes are available.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4908, "Announces when terminal Quick Fixes are available.")),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.terminalBell': {
            ...signalFeatureBase,
            'description': ( localize(
                4909,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when the terminal bell is ringing."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4910, "Plays a sound when the terminal bell is ringing.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4911, "Announces when the terminal bell is ringing.")),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.diffLineInserted': {
            ...defaultNoAnnouncement,
            'description': ( localize(
                4912,
                "Plays a sound / audio cue when the focus moves to an inserted line in Accessible Diff Viewer mode or to the next/previous change."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4913,
                        "Plays a sound when the focus moves to an inserted line in Accessible Diff Viewer mode or to the next/previous change."
                    )),
                    ...soundFeatureBase
                }
            }
        },
        'accessibility.signals.diffLineModified': {
            ...defaultNoAnnouncement,
            'description': ( localize(
                4914,
                "Plays a sound / audio cue when the focus moves to an modified line in Accessible Diff Viewer mode or to the next/previous change."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4915,
                        "Plays a sound when the focus moves to a modified line in Accessible Diff Viewer mode or to the next/previous change."
                    )),
                    ...soundFeatureBase
                }
            }
        },
        'accessibility.signals.diffLineDeleted': {
            ...defaultNoAnnouncement,
            'description': ( localize(
                4916,
                "Plays a sound / audio cue when the focus moves to an deleted line in Accessible Diff Viewer mode or to the next/previous change."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4917,
                        "Plays a sound when the focus moves to an deleted line in Accessible Diff Viewer mode or to the next/previous change."
                    )),
                    ...soundFeatureBase
                }
            }
        },
        'accessibility.signals.notebookCellCompleted': {
            ...signalFeatureBase,
            'description': ( localize(
                4918,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a notebook cell execution is successfully completed."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(
                        4919,
                        "Plays a sound when a notebook cell execution is successfully completed."
                    )),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(
                        4920,
                        "Announces when a notebook cell execution is successfully completed."
                    )),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.notebookCellFailed': {
            ...signalFeatureBase,
            'description': ( localize(
                4921,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a notebook cell execution fails."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4922, "Plays a sound when a notebook cell execution fails.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4923, "Announces when a notebook cell execution fails.")),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.chatRequestSent': {
            ...signalFeatureBase,
            'description': ( localize(
                4924,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a chat request is made."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4925, "Plays a sound when a chat request is made.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4926, "Announces when a chat request is made.")),
                    ...announcementFeatureBase
                },
            }
        },
        'accessibility.signals.progress': {
            ...signalFeatureBase,
            'description': ( localize(
                4927,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - on loop while progress is occurring."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4928, "Plays a sound on loop while progress is occurring.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4929, "Alerts on loop while progress is occurring.")),
                    ...announcementFeatureBase
                },
            },
        },
        'accessibility.signals.chatResponseReceived': {
            ...defaultNoAnnouncement,
            'description': ( localize(4930, "Plays a sound / audio cue when the response has been received.")),
            'properties': {
                'sound': {
                    'description': ( localize(4931, "Plays a sound on loop while the response has been received.")),
                    ...soundFeatureBase
                },
            }
        },
        'accessibility.signals.voiceRecordingStarted': {
            ...defaultNoAnnouncement,
            'description': ( localize(4932, "Plays a sound / audio cue when the voice recording has started.")),
            'properties': {
                'sound': {
                    'description': ( localize(4933, "Plays a sound when the voice recording has started.")),
                    ...soundFeatureBase,
                },
            },
            'default': {
                'sound': 'on'
            }
        },
        'accessibility.signals.voiceRecordingStopped': {
            ...defaultNoAnnouncement,
            'description': ( localize(4934, "Plays a sound / audio cue when the voice recording has stopped.")),
            'properties': {
                'sound': {
                    'description': ( localize(4935, "Plays a sound when the voice recording has stopped.")),
                    ...soundFeatureBase,
                    default: 'off'
                },
            }
        },
        'accessibility.signals.clear': {
            ...signalFeatureBase,
            'description': ( localize(
                4936,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a feature is cleared (for example, the terminal, Debug Console, or Output channel)."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4937, "Plays a sound when a feature is cleared.")),
                    ...soundFeatureBase
                },
                'announcement': {
                    'description': ( localize(4938, "Announces when a feature is cleared.")),
                    ...announcementFeatureBase
                },
            },
        },
        'accessibility.signals.save': {
            'type': 'object',
            'tags': ['accessibility'],
            additionalProperties: false,
            'markdownDescription': ( localize(
                4939,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a file is saved."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4940, "Plays a sound when a file is saved.")),
                    'type': 'string',
                    'enum': ['userGesture', 'always', 'never'],
                    'default': 'never',
                    'enumDescriptions': [
                        ( localize(4941, "Plays the sound when a user explicitly saves a file.")),
                        ( localize(4942, "Plays the sound whenever a file is saved, including auto save.")),
                        ( localize(4943, "Never plays the sound."))
                    ],
                },
                'announcement': {
                    'description': ( localize(4944, "Announces when a file is saved.")),
                    'type': 'string',
                    'enum': ['userGesture', 'always', 'never'],
                    'default': 'never',
                    'enumDescriptions': [
                        ( localize(4945, "Announces when a user explicitly saves a file.")),
                        ( localize(4946, "Announces whenever a file is saved, including auto save.")),
                        ( localize(4947, "Never plays the announcement."))
                    ],
                },
            },
            default: {
                'sound': 'never',
                'announcement': 'never'
            }
        },
        'accessibility.signals.format': {
            'type': 'object',
            'tags': ['accessibility'],
            additionalProperties: false,
            'markdownDescription': ( localize(
                4948,
                "Plays a signal - sound (audio cue) and/or announcement (alert) - when a file or notebook is formatted."
            )),
            'properties': {
                'sound': {
                    'description': ( localize(4949, "Plays a sound when a file or notebook is formatted.")),
                    'type': 'string',
                    'enum': ['userGesture', 'always', 'never'],
                    'default': 'never',
                    'enumDescriptions': [
                        ( localize(4950, "Plays the sound when a user explicitly formats a file.")),
                        ( localize(
                            4951,
                            "Plays the sound whenever a file is formatted, including if it is set to format on save, type, or, paste, or run of a cell."
                        )),
                        ( localize(4952, "Never plays the sound."))
                    ],
                },
                'announcement': {
                    'description': ( localize(4953, "Announces when a file or notebook is formatted.")),
                    'type': 'string',
                    'enum': ['userGesture', 'always', 'never'],
                    'default': 'never',
                    'enumDescriptions': [
                        ( localize(4954, "Announces when a user explicitly formats a file.")),
                        ( localize(
                            4955,
                            "Announces whenever a file is formatted, including if it is set to format on save, type, or, paste, or run of a cell."
                        )),
                        ( localize(4956, "Never announces."))
                    ],
                },
            },
            default: {
                'sound': 'never',
                'announcement': 'never'
            }
        },
        'accessibility.underlineLinks': {
            'type': 'boolean',
            'description': ( localize(4957, "Controls whether links should be underlined in the workbench.")),
            'default': false,
        },
        'accessibility.debugWatchVariableAnnouncements': {
            'type': 'boolean',
            'description': ( localize(
                4958,
                "Controls whether variable changes should be announced in the debug watch view."
            )),
            'default': true,
        },
    }
};
function registerAccessibilityConfiguration() {
    const registry = ( (Registry.as(Extensions$1.Configuration)));
    registry.registerConfiguration(configuration);
    registry.registerConfiguration({
        ...workbenchConfigurationNodeBase,
        properties: {
            ["accessibility.dimUnfocused.enabled" ]: {
                description: ( localize(
                    4959,
                    'Whether to dim unfocused editors and terminals, which makes it more clear where typed input will go to. This works with the majority of editors with the notable exceptions of those that utilize iframes like notebooks and extension webview editors.'
                )),
                type: 'boolean',
                default: false,
                tags: ['accessibility'],
                scope: 1 ,
            },
            ["accessibility.dimUnfocused.opacity" ]: {
                markdownDescription: ( localize(
                    4960,
                    'The opacity fraction (0.2 to 1.0) to use for unfocused editors and terminals. This will only take effect when {0} is enabled.',
                    `\`#${"accessibility.dimUnfocused.enabled" }#\``
                )),
                type: 'number',
                minimum: 0.2 ,
                maximum: 1 ,
                default: 0.75 ,
                tags: ['accessibility'],
                scope: 1 ,
            },
            ["accessibility.hideAccessibleView" ]: {
                description: ( localize(4961, "Controls whether the Accessible View is hidden.")),
                type: 'boolean',
                default: false,
                tags: ['accessibility']
            }
        }
    });
}
const SpeechTimeoutDefault = 1200;
let DynamicSpeechAccessibilityConfiguration = class DynamicSpeechAccessibilityConfiguration extends Disposable {
    static { this.ID = 'workbench.contrib.dynamicSpeechAccessibilityConfiguration'; }
    constructor(speechService, productService) {
        super();
        this.speechService = speechService;
        this.productService = productService;
        this._register(Event.runAndSubscribe(speechService.onDidChangeHasSpeechProvider, () => this.updateConfiguration()));
    }
    updateConfiguration() {
        if (!this.speechService.hasSpeechProvider) {
            return;
        }
        const languages = this.getLanguages();
        const languagesSorted = ( (Object.keys(languages))).sort((langA, langB) => {
            return languages[langA].name.localeCompare(languages[langB].name);
        });
        const registry = ( (Registry.as(Extensions$1.Configuration)));
        registry.registerConfiguration({
            ...accessibilityConfigurationNodeBase,
            properties: {
                ["accessibility.voice.speechTimeout" ]: {
                    'markdownDescription': ( localize(
                        4962,
                        "The duration in milliseconds that voice speech recognition remains active after you stop speaking. For example in a chat session, the transcribed text is submitted automatically after the timeout is met. Set to `0` to disable this feature."
                    )),
                    'type': 'number',
                    'default': SpeechTimeoutDefault,
                    'minimum': 0,
                    'tags': ['accessibility']
                },
                ["accessibility.voice.speechLanguage" ]: {
                    'markdownDescription': ( localize(
                        4963,
                        "The language that text-to-speech and speech-to-text should use. Select `auto` to use the configured display language if possible. Note that not all display languages maybe supported by speech recognition and synthesizers."
                    )),
                    'type': 'string',
                    'enum': languagesSorted,
                    'default': 'auto',
                    'tags': ['accessibility'],
                    'enumDescriptions': ( (languagesSorted.map(key => languages[key].name))),
                    'enumItemLabels': ( (languagesSorted.map(key => languages[key].name)))
                },
                ["accessibility.voice.autoSynthesize" ]: {
                    'type': 'string',
                    'enum': ['on', 'off', 'auto'],
                    'enumDescriptions': [
                        ( localize(
                            4964,
                            "Enable the feature. When a screen reader is enabled, note that this will disable aria updates."
                        )),
                        ( localize(4965, "Disable the feature.")),
                        ( localize(
                            4966,
                            "When a screen reader is detected, disable the feature. Otherwise, enable the feature."
                        ))
                    ],
                    'markdownDescription': ( localize(
                        4967,
                        "Whether a textual response should automatically be read out aloud when speech was used as input. For example in a chat session, a response is automatically synthesized when voice was used as chat request."
                    )),
                    'default': this.productService.quality !== 'stable' ? 'auto' : 'off',
                    'tags': ['accessibility']
                }
            }
        });
    }
    getLanguages() {
        return {
            ['auto']: {
                name: ( localize(4968, "Auto (Use Display Language)"))
            },
            ...SPEECH_LANGUAGES
        };
    }
};
DynamicSpeechAccessibilityConfiguration = ( (__decorate([
    ( (__param(0, ISpeechService))),
    ( (__param(1, IProductService)))
], DynamicSpeechAccessibilityConfiguration)));
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations([{
        key: 'audioCues.volume',
        migrateFn: (value, accessor) => {
            return [
                ['accessibility.signalOptions.volume', { value }],
                ['audioCues.volume', { value: undefined }]
            ];
        }
    }]);
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations([{
        key: 'audioCues.debouncePositionChanges',
        migrateFn: (value) => {
            return [
                ['accessibility.signalOptions.debouncePositionChanges', { value }],
                ['audioCues.debouncePositionChanges', { value: undefined }]
            ];
        }
    }]);
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations([{
        key: 'accessibility.signalOptions',
        migrateFn: (value, accessor) => {
            const delayGeneral = getDelaysFromConfig(accessor, 'general');
            const delayError = getDelaysFromConfig(accessor, 'errorAtPosition');
            const delayWarning = getDelaysFromConfig(accessor, 'warningAtPosition');
            const volume = getVolumeFromConfig(accessor);
            const debouncePositionChanges = getDebouncePositionChangesFromConfig(accessor);
            const result = [];
            if (!!volume) {
                result.push(['accessibility.signalOptions.volume', { value: volume }]);
            }
            if (!!delayGeneral) {
                result.push(['accessibility.signalOptions.experimental.delays.general', { value: delayGeneral }]);
            }
            if (!!delayError) {
                result.push(['accessibility.signalOptions.experimental.delays.errorAtPosition', { value: delayError }]);
            }
            if (!!delayWarning) {
                result.push(['accessibility.signalOptions.experimental.delays.warningAtPosition', { value: delayWarning }]);
            }
            if (!!debouncePositionChanges) {
                result.push(['accessibility.signalOptions.debouncePositionChanges', { value: debouncePositionChanges }]);
            }
            result.push(['accessibility.signalOptions', { value: undefined }]);
            return result;
        }
    }]);
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations([{
        key: 'accessibility.signals.sounds.volume',
        migrateFn: (value) => {
            return [
                ['accessibility.signalOptions.volume', { value }],
                ['accessibility.signals.sounds.volume', { value: undefined }]
            ];
        }
    }]);
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations([{
        key: 'accessibility.signals.debouncePositionChanges',
        migrateFn: (value) => {
            return [
                ['accessibility.signalOptions.debouncePositionChanges', { value }],
                ['accessibility.signals.debouncePositionChanges', { value: undefined }]
            ];
        }
    }]);
function getDelaysFromConfig(accessor, type) {
    return accessor(`accessibility.signalOptions.experimental.delays.${type}`) || accessor('accessibility.signalOptions')?.['experimental.delays']?.[`${type}`] || accessor('accessibility.signalOptions')?.['delays']?.[`${type}`];
}
function getVolumeFromConfig(accessor) {
    return accessor('accessibility.signalOptions.volume') || accessor('accessibility.signalOptions')?.volume || accessor('accessibility.signals.sounds.volume') || accessor('audioCues.volume');
}
function getDebouncePositionChangesFromConfig(accessor) {
    return accessor('accessibility.signalOptions.debouncePositionChanges') || accessor('accessibility.signalOptions')?.debouncePositionChanges || accessor('accessibility.signals.debouncePositionChanges') || accessor('audioCues.debouncePositionChanges');
}
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations([{
        key: "accessibility.voice.autoSynthesize" ,
        migrateFn: (value) => {
            let newValue;
            if (value === true) {
                newValue = 'on';
            }
            else if (value === false) {
                newValue = 'off';
            }
            else {
                return [];
            }
            return [
                ["accessibility.voice.autoSynthesize" , { value: newValue }],
            ];
        }
    }]);
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations([{
        key: 'accessibility.signals.chatResponsePending',
        migrateFn: (value, accessor) => {
            return [
                ['accessibility.signals.progress', { value }],
                ['accessibility.signals.chatResponsePending', { value: undefined }],
            ];
        }
    }]);
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations(( (AccessibilitySignal.allAccessibilitySignals.map(item => item.legacySoundSettingsKey ? ({
    key: item.legacySoundSettingsKey,
    migrateFn: (sound, accessor) => {
        const configurationKeyValuePairs = [];
        const legacyAnnouncementSettingsKey = item.legacyAnnouncementSettingsKey;
        let announcement;
        if (legacyAnnouncementSettingsKey) {
            announcement = accessor(legacyAnnouncementSettingsKey) ?? undefined;
            if (announcement !== undefined && typeof announcement !== 'string') {
                announcement = announcement ? 'auto' : 'off';
            }
        }
        configurationKeyValuePairs.push([`${item.legacySoundSettingsKey}`, { value: undefined }]);
        configurationKeyValuePairs.push([`${item.settingsKey}`, { value: announcement !== undefined ? { announcement, sound } : { sound } }]);
        return configurationKeyValuePairs;
    }
}) : undefined))).filter(isDefined));
( (Registry.as(Extensions.ConfigurationMigration)))
    .registerConfigurationMigrations(( (AccessibilitySignal.allAccessibilitySignals.filter(i => !!i.legacyAnnouncementSettingsKey && !!i.legacySoundSettingsKey).map(item => ({
    key: item.legacyAnnouncementSettingsKey,
    migrateFn: (announcement, accessor) => {
        const configurationKeyValuePairs = [];
        const sound = accessor(item.settingsKey)?.sound || accessor(item.legacySoundSettingsKey);
        if (announcement !== undefined && typeof announcement !== 'string') {
            announcement = announcement ? 'auto' : 'off';
        }
        configurationKeyValuePairs.push([`${item.settingsKey}`, { value: announcement !== undefined ? { announcement, sound } : { sound } }]);
        configurationKeyValuePairs.push([`${item.legacyAnnouncementSettingsKey}`, { value: undefined }]);
        configurationKeyValuePairs.push([`${item.legacySoundSettingsKey}`, { value: undefined }]);
        return configurationKeyValuePairs;
    }
})))));
export { DynamicSpeechAccessibilityConfiguration, SpeechTimeoutDefault, accessibilityConfigurationNodeBase, accessibilityHelpIsShown, accessibleViewContainsCodeBlocks, accessibleViewCurrentProviderId, accessibleViewGoToSymbolSupported, accessibleViewHasAssignedKeybindings, accessibleViewHasUnassignedKeybindings, accessibleViewInCodeBlock, accessibleViewIsShown, accessibleViewOnLastLine, accessibleViewSupportsNavigation, accessibleViewVerbosityEnabled, announcementFeatureBase, registerAccessibilityConfiguration, soundFeatureBase };
